# This is my API key for custom search: AIzaSyBtvtZLfZuC6PbgOGmp0DNsLlJx5qZyX1s

import sys
from copy import deepcopy
from random import shuffle, choices, randint
import requests
import re
from bs4 import BeautifulSoup
from googleapiclient.discovery import build

class Crawler:
    def __init__(self, query="query", mode=0):
        self.query = query
        self.mode = mode

        #random result of search to use
        self.randomSearchIndex = 0

        # this builds a service to interact with the google custom search api
        self.service = build("customsearch", "v1", developerKey="AIzaSyBtvtZLfZuC6PbgOGmp0DNsLlJx5qZyX1s")
    def runCrawl(self):
        pass

    def verifySearchResults(self, result):
        """Verifies that a given search has results to send back. If not, program exits with error code 1 (no results)"""
        totalResults = result['queries']['request'][0]['totalResults']
        if int(totalResults) < 1:
            print("Error, no results found for search term", file=sys.stderr)
            exit(1)


class TextCrawler(Crawler):
    def __init__(self, query="query", mode=0):
        super().__init__(query,mode)

    def runCrawl(self):
        self.searchResults = self.textSearch(self.query)
        self.randomSearchIndex = randint(0, len(self.searchResults)-1)
        self.mediaFound = self.textCrawl(self.searchResults[self.randomSearchIndex]['link'])
        return self.mediaFound

    def buildArticleText(self, text):
        temp = deepcopy(text)
        temp = choices(temp, k=randint(1, len(temp)))
        tempSet = set(temp)
        preText = list(tempSet)
        shuffle(preText)
        articleText = '\n'.join(preText)
        return articleText

    def fixLinksText(self, stringToFix, originalURL):
        """This is used to fix links that are given in text, as they are passed as relative links, in order to work properly
            they are converted to absolute addresses

            Citations links are also removed as they point to data that isn't scraped, as are 'citation needed' sups"""
        if 'href=' in stringToFix:
            try:
                # fix links
                trimmedURL = originalURL[:originalURL.index('/wiki')]
                fixedString = stringToFix.replace('href="/wiki', 'href="' + trimmedURL + '/wiki')

                # remove citation links
                pattern1 = re.compile(r'<sup class="reference".*</sup>')
                fixedString = pattern1.sub('', fixedString)

                pattern2 = re.compile(r'<sup class ="noprint.*".*</sup>')
                fixedString = pattern2.sub('', fixedString)
                return fixedString

            except:
                print("fixLinksText failed to fix link: ", originalURL, file=sys.stderr)
                return stringToFix
        else:
            return stringToFix

    def textSearch(self, query):
        # this builds a service to interact with the google custom search api
        service = build("customsearch", "v1",
                        developerKey="AIzaSyBtvtZLfZuC6PbgOGmp0DNsLlJx5qZyX1s")

        res = service.cse().list(
            q=query,
            cx='008445415007179572432:nww82heqndm', #what search engine to use
            start=randint(1, 50),  # note, a value of 0 appears to cause a bad request error
            safe='medium' #level of safe search enabled
        ).execute()
        self.verifySearchResults(res)
        # pprint.pprint(res)
        return res['items']

    def textCrawl(self, url):
        """Given a url this method retrieves the data from that source. It then pulls out all text in <p> tags, passes it to
            another method to fix inconsistencies, and then returns all the data as a single string"""
        doc = requests.get(url)
        soup = BeautifulSoup(doc.text, 'html.parser')

        text = []
        for string in soup.find_all('p'):
            # fixes inconsistencies in links, etc.
            fixedString = self.fixLinksText(str(string), url)
            text.append(fixedString)

        articleText = self.buildArticleText(text)

        # print(text)
        if articleText:
            return "<a href='"+url+"'>"+url+"</a> <br>" + articleText
        else:
            print("error converting crawled text to article text", file=sys.stderr)

class ImageCrawler(Crawler):
    def __init__(self, query="query", mode=0):
        super().__init__(query,mode)

    def runCrawl(self):
        self.searchResults = self.imageSearch(self.query)
        self.randomSearchIndex = randint(0, len(self.searchResults)-1)
        self.mediaFound = self.imageCrawl(self.searchResults[self.randomSearchIndex]['link'])
        return self.mediaFound

    def imageSearch(self, query):
        # this builds a service to interact with the google custom search api
        service = build("customsearch", "v1",
                        developerKey="AIzaSyBtvtZLfZuC6PbgOGmp0DNsLlJx5qZyX1s")

        res = service.cse().list(
            q=query,
            cx='008445415007179572432:3ar3-1szfjm',
            start=randint(1, 50),
            searchType="image",
            safe='medium'
        ).execute()
        self.verifySearchResults(res)
        # pprint.pprint(res)
        return res['items']

    def imageCrawl(self, url):
        """Given a url this method retrieves the data for a picture from it and returns it this is both to preserve network use
            and to cover myself (better) legally"""

        # print(text)
        return " <a href='"+url+"'>"+url+"</a> <br> <img src='" + url + "' alt='I'm a picture!'>"

class VideoCrawler(Crawler):
    def __init__(self, query="query", mode=0):
        super().__init__(query,mode)

    def runCrawl(self):
        self.searchResults = self.videoSearch(self.query)
        self.randomSearchIndex = randint(0, len(self.searchResults)-1)
        self.mediaFound = self.videoCrawl(self.searchResults[self.randomSearchIndex]['link'])
        return self.mediaFound

    def videoSearch(self,query):
        res = self.service.cse().list(
            q=query,
            cx='008445415007179572432:4wsb9ozcmtc',
            start=randint(1, 50),
            safe='medium'
        ).execute()
        self.verifySearchResults(res)
        return res['items']

    def videoCrawl(self, url):
        """Given a url this method retrieves takes it and formats it into an 'a' tag for html to be passed back"""
        pattern = re.compile(r'.*\?v=')
        videoCode = pattern.sub('', url)

        embed='<a href='+url+'>'+url+'</a> <br> <iframe width = "560" height = "315" src = "https://www.youtube.com/embed/'+videoCode+'" '
        embed += 'frameborder = "0" allow = "autoplay; encrypted-media allowfullscreen"> </iframe>'

        #return "<a href='" + url + "'>Video Found! </a>"
        return embed

if __name__ == '__main__':
    #list of function pointers to constructors
    constructors =[TextCrawler,ImageCrawler,VideoCrawler]

    #Denotes whether this was a manual launch, or a command line launch (defaults True)
    CL = True

    #checks if program is supposed to run autonomously or with user input
    if len(sys.argv) == 3:
        mediaType = int(sys.argv[1])
        query = sys.argv[2]
    else:
        query = input('What would you like to search?')
        mediaType = int(input('What Mode?'))
        CL = False

    crawler = constructors[mediaType](query=query,mode=mediaType)
    mediaFound = crawler.runCrawl()

    #in case of a command line call we should return data, otherwise write to a file to debug
    if(CL):
        print(mediaFound)
    else:
        with open("test.html", mode='wb+') as f:
            f.write(mediaFound.encode('UTF-16'))