var cmd=require('node-cmd');
const fs = require('fs');
var uuid4 = require('uuid4');
var uuidValid = require('uuid-validate');
var ip = require('ip');

//my own classes
var sqlConnector = require('./mysqlConnector');
var socketIOClient = require('./socketIOClient');
var socketIOServer = require('./socketIOServer');

class Hub{
    constructor(){
        this.uuid = uuid4();
        this.activeConnections = {};
        this.knownMainHub = 0;
        this.ipAddr = ip.address();

        this.mysqlConn = new sqlConnector.MySqlConnector(this);
        this.verifyHubCreation();

        //sets state to auxillary hub, then launches startup
        this.state = auxHub(this);
        this.state.startup();
        var hub = this;

        this.dispatchSearch = function(searchQuery, mode, mobileID){
            hub.state.dispatchSearch(searchQuery, mode, mobileID)
        };

        this.runCrawl = function(data, IDs) {
            console.log("run crawl for: ",data,IDs);
            hub.state.runCrawl(data, IDs);
        };

        this.checkInetConn = function(){
            require('dns').lookup("euclid.nmu.edu",function (err) {
                if(err){
                    console.log("Fatal error: No internet connection detected, exiting...");
                    throw err;
                }
            })
        };

        return this;
    }

    uuidValid(){
        return uuidValid(this.uuid);
    }

    addActiveConn(key, value){
        this.activeConnections[key] = value;
    }

    removeActiveConn(key){
        delete this.activeConnections[key];
    }

    verifyHubCreation(){
        if(!this.uuidValid()){
            //bad uuid. exit code 1
            process.exit(1)
        }
    }
}

function mainHub(hub){
    this.hub = hub;
    hub.IDPairs = {};
    var storedResults = {};

    this.startup(hub);

    var bootMessage = "Running start up for Main hub...";

    this.startup = function (hub) {
        this.hub = hub;
        console.log(bootMessage);
        hub.mysqlConn.addMainHub(hub.uuid, hub.ipAddr);
        //removeSelfFromActiveConns();
        startServer();
        //hub.state.dispatchSearch("cat", "123456123456123456123456123456123456");
    };

    this.pairIDs = function(socketID, uuid){
        hub.IDPairs[socketID] = uuid;
    };

    this.removeActiveConn = function(socketID){
        var hubUUID = hub.IDPairs[socketID];

        hub.mysqlConn.removeActiveConn(hubUUID);
        delete hub.IDPairs[socketID];
    };

    this.save = function(mobileID){
        if(storedResults[mobileID] != undefined) {
            var data = storedResults[mobileID][0];
            var term = storedResults[mobileID][1]
            hub.mysqlConn.save(mobileID, term, data);
        }
    };

    this.loadList = function(mobileID){
        hub.mysqlConn.loadList(mobileID)
    };

    this.loadArt = function(mobileID,term){
        hub.mysqlConn.loadArt(mobileID,term)
    }

    this.loadedList = function (mobileID, results) {
        hub.server.loadedList(mobileID, results);
    };

    function returnSearchResults(key,mobileID) {
        console.log("returning search Results for " + mobileID);
        hub.server.sendSearchResults(storedResults[key][0],mobileID);
    }

    this.recordResults = function(data, IDs){
        var key = IDs[1];
        if(storedResults[key] == undefined || storedResultsDone(key) || storedResults[key][1] != data[0]){
            storedResults[key] = [[undefined,undefined,undefined],data[0]];
        }

        storedResults[key][0][data[1]] = data[2];

        if(storedResultsDone(key)){
            returnSearchResults(key, IDs[1]);
        }
    };

    function storedResultsDone(key){
        return storedResults[key][0][0] != undefined && storedResults[key][0][1] != undefined && storedResults[key][0][2] != undefined
    }

    function removeSelfFromActiveConns(){
        hub.mysqlConn.removeActiveConn(hub.uuid);
    };

    this.dispatchSearch = function(searchQuery, mobileID){
        hub.server.dispatchSearch(searchQuery, mobileID)
    };

    function startServer() {
        hub.server = new socketIOServer.SocketIOServer(hub);
    }

    this.selfSearch = function (searchQuery, mobileID){
        this.runCrawl([searchQuery,0], [,mobileID])
        this.runCrawl([searchQuery,1], [,mobileID])
        this.runCrawl([searchQuery,2], [,mobileID])
    };


    function crawlResults(query, mode, results, errors, IDs){
            if(errors){
                this.recordResults([query,mode,errors], IDs);
            }
            else{
                this.recordResults([query,mode,results], IDs);
            }
    }

    this.runCrawl = function(data, IDs) {
        //given a string query, and an int mode runs a crawler, and calls a return function
        cmd.get(
            'python3 ../Crawler.py ' + data[1].toString() + ' "' + data[0] + '"',
            function (err, res, stderr) {
                crawlResults(data[0], data[1], res, stderr, IDs)
            }
        );
    };
    return this;
}



function auxHub(hub){
    this.hub = hub;
    var bootMessage = "Running start up for Auxillary hub...";
    var switchMessage = "Switching from Auxillary to Main Hub...";
    var deathMessage = "Critical Error. See you, space cowboy.";

    this.startup = function () {
        console.log(bootMessage);
        addSelfToKnownConns(retrieveKnownHubs);
    };
    function addSelfToKnownConns(callback){
        hub.mysqlConn.addActiveConn(hub.uuid,hub.ipAddr, callback);
    }

    function updateActiveConns(results){
        this.hub.activeConnections = results;
        retrieveMainHub();
    }

    function updateMainConnLong(results){
        this.hub.knownMainHub = results;
        determineRole();
    }

    function updateMainConnShort(results){
        this.hub.knownMainHub = results;
    }

    function activeConnsUpdated(){
        this.retrieveMainHub();
    };

    function retrieveMainHub(){
        this.hub.mysqlConn.activeMainHub(hub, updateMainConnLong)
    };

    function retrieveKnownHubs() {
        setTimeout(function() {
            hub.mysqlConn.activeConnections(hub, updateActiveConns);
        },1000);
    };

    function determineRole() {
        //if there is already a main Hub, we should just connect
        if (this.hub.knownMainHub.length !== 0){
            connectToMainHub();
        }
        else{
            determineMainEligibile();
        }
    };

    function determineMainEligibile() {
        console.log("no main hub found, determining if eligible to be new main...");

        this.hub.mysqlConn.activeMainHub(this.hub, updateMainConnShort);

        setTimeout(function () {

        }, 1000);

        if(hub.activeConnections.length == 0){
            addSelfToKnownConns();
            retrieveKnownHubs();
        }
        //since all hardware and software is equivalent, eligibility will be determined arbitrarily
        // (in this case by the alphabetical order of uuid)
        else if(hub.activeConnections[0][0] == hub.uuid && hub.knownMainHub.length == 0){
            console.log("elligible for upgrade to Main hub");
            switchRoles();
        }
        else{
            console.log("not elligible for upgrade to Main hub");
            connectToMainHub(0);
        }
    };

    function switchRoles(){
        console.log(switchMessage);
        if(typeof hub.client !== 'undefined'){
            delete hub.client;
        }
        hub.state = mainHub(this.hub);
    };

    function connectToMainHub(attempt){
        if(this.hub.knownMainHub.length === 0 ) {
            this.hub.mysqlConn.activeMainHub(this.hub, updateMainConnShort);
            setTimeout(function () {

                if(attempt > 3){
                    console.log("Too many attempts to find main hub, restructuring...");
                    //TODO: handle new main not appearing, i'm thinking delete top result of activeConns from db
                    hub.mysqlConn.removeActiveConn(hub.activeConnections[0][0]);
                    retrieveKnownHubs();
                }
                else{
                    console.log("Could not determine main hub, trying again...");
                    connectToMainHub(attempt+1);
                }
            }, 1000);
        }
        else {
            console.log("Main hub determined at "+ this.hub.knownMainHub[0][1] +" connecting...");
            this.hub.client = new socketIOClient.SocketIOClient(this.hub, this.hub.knownMainHub[0][1]);
        }
    };

    function deleteKnownMainHub(){
        if (hub.knownMainHub.length != 0) {
            console.log("Removing uuid ", hub.knownMainHub[0][0], " from database...");
            hub.mysqlConn.removeMainConn(hub.knownMainHub[0][0]);
            hub.mysqlConn.removeActiveConn(hub.knownMainHub[0][0]);
        }

        retrieveKnownHubs();
    }

    this.handleTimouts = function(){
        var currentMain = this.hub.knownMainHub;
        hub.mysqlConn.activeMainHub(this.hub, updateMainConnShort);
        setTimeout(function () {
            if(currentMain.length != 0 && currentMain[0][0] === currentMain[0][0]){
                deleteKnownMainHub()
            }
            else{
                retrieveKnownHubs();
            }
        }, 250);
    };

    this.runCrawl = function(data, IDs) {
        //given a string query, and an int mode runs a crawler, and calls a return function
        cmd.get(
            'python3 ../Crawler.py ' + data[1].toString() + ' "' + data[0] + '"',
            function (err, res, stderr) {
                crawlResults(data[0],data[1],res, stderr, IDs)
            }
        );
    };

    function crawlResults(query, mode,results, errors, IDs){
        if(errors){
            hub.client.returnResults([query,mode,errors], IDs);
        }
        else{
            hub.client.returnResults([query,mode,results], IDs);
        }
    }

    return this;
}

var testHub = new Hub();

module.exports.Hub = Hub;
