var app = require('express')();
var server = require('http').Server(app);
var io = require('socket.io')(server);

class SocketIOServer{
    constructor(hub){
        var hub = hub;
        console.log("Launching socketIO server");
        var crawlTurn = 0;
        var openConns = [];
        var mobileIDPairs = {};

        server.listen(4321);

        var mobile = io.of('/mobile');
        mobile.on('connection', function (socket) {
            console.log("mobile connection detected: ", socket.id);

            socket.on('recieveSearchRequest',function (searchQuery, mobileID) {
                hub.dispatchSearch(searchQuery, mobileID);
            });

            socket.on('receiveUUID', function(mobileID){
                pairMobileIDs(socket.id, mobileID);
            });

            socket.on('disconnect', function () {
                //removeActiveConn(socket.id)
                console.log('mobile disconnected', socket.id )
            });

            socket.on('receiveLoadListRequest', function (mobileID) {
                console.log("Mobile user "+mobileID+" requested list of saves.");
                hub.state.loadList(mobileID)
            });

            socket.on('receiveLoadArt', function (mobileID, term) {
                console.log("Mobile user "+mobileID+" requested to load an article about: " + term);
                hub.state.loadArt(mobileID,term)
            });

            socket.on('receiveSaveRequest', function (mobileID) {
                console.log("Request to save article from: ", mobileID)
                hub.state.save(mobileID);
            })
        })

        var aux = io.of('/auxHub');
        aux.on('connection', function (socket) {
            var socketAddress = socket.handshake;
            var ipAddress = socketAddress.address.split(":")[3];

            recordConnection(ipAddress,socket.id);

            socket.emit('requestUUID');

            socket.on('recieveCrawlResult',function (data, IDs) {
                recordCrawlResults(data,IDs);
            });

            socket.on('recieveUUID',function (uuid) {
                pairIDs(socket.id, uuid);
            });

            socket.on('disconnect', function () {
                console.log('auxHub disconnected',socket.id)
                hub.checkInetConn()
            });

        });

        function pairMobileIDs(socketID, mobileID){
            mobileIDPairs[mobileID] = socketID;
        }

        function getMobileSocketID(mobileID){
            return mobileIDPairs[mobileID];
        }
        
        this.dispatchSearch = function(searchQuery, mobileID) {
            var connSockets = connectedSockets();
            var numberOfConnectedSockets = connectedSockets().length;
            if(numberOfConnectedSockets == 0){
                hub.state.selfSearch(searchQuery, mobileID);
            }
            else{
                if(crawlTurn > numberOfConnectedSockets){
                    crawlTurn = 0;
                }
                //runs 3 crawls
                requestCrawl([searchQuery,0], connSockets[crawlTurn], mobileID)
                crawlTurn = (crawlTurn + 1) % numberOfConnectedSockets;
                requestCrawl([searchQuery,1], connSockets[crawlTurn], mobileID)
                crawlTurn = (crawlTurn + 1) % numberOfConnectedSockets;
                requestCrawl([searchQuery,2], connSockets[crawlTurn], mobileID)
                crawlTurn = (crawlTurn + 1) % numberOfConnectedSockets;
            }

            console.log("dispatching search request...");
        };

        this.sendSearchResults = function(data, mobileID){
            var finalData = {}
            finalData["text"] = data[0];
            finalData["pic"] = data[1];
            finalData["vid"] = data[2];
            var sockID = getMobileSocketID(mobileID);
            setTimeout(function(){},250);
            console.log("sending completed search request on to" + mobileID);
            mobile.to(sockID).emit('returnSearchResults',finalData);
            //hub.state.save(mobileID);
        };

        this.loadedList = function (mobileID, results) {
            var sockID = getMobileSocketID(mobileID);
            setTimeout(function(){},250);
            console.log("sending out list of saves", mobileID);
            mobile.to(sockID).emit('returnSaveList',results);
        };

        function requestCrawl(queryTerm, socketID, mobileID){
            aux.to(socketID).emit('crawlFor',queryTerm, [socketID, mobileID]);
            console.log("requested crawl for", socketID, queryTerm )
        }

        function recordCrawlResults(data, IDs){
            hub.state.recordResults(data, IDs);
            console.log("recording returned result");
        }

        function removeActiveConn(socketID){
            hub.state.removeActiveConn(socketID);
        }
        
        function pairIDs(socketID, uuid) {
            hub.state.pairIDs(socketID, uuid);
        }

        function deleteConn(ip, socketid){
            console.log("delete: ",ip,socketid);
        }

        function recordConnection(ip, id){
            console.log('Connected:',ip, id);
        }

        function connectedSockets(){
            //console.log("connected clients", io.of('/').in('auxHub').clients());
            var list = io.of('/auxHub').clients();
            var retVal = [];
            Object.keys(list.sockets).forEach(function (element) {
                retVal.push(element);
            });
            return retVal;
        }
        return this;
    }

}

module.exports.SocketIOServer = SocketIOServer;
//var test = new SocketIOServer(0);



